@@ -845,11 +845,18 @@ int ltdb_modify_internal(struct ldb_module *module,
 			if (idx != -1) {
 				j = (unsigned int) idx;
 				el2 = &(msg2->elements[j]);
-				if (ldb_msg_element_compare(el, el2) == 0) {
-					/* we are replacing with the same values */
+
+				/* we consider two elements to be
+				 * equal only if the order
+				 * matches. This allows dbcheck to
+				 * fix the ordering on attributes
+				 * where order matters, such as
+				 * objectClass
+				 */
+				if (ldb_msg_element_equal_ordered(el, el2)) {
 					continue;
 				}
-			
+
 				/* Delete the attribute if it exists in the DB */
 				if (msg_delete_attribute(module, ldb, msg2,
 							 el->name) != 0) {
@@ -1418,14 +1425,8 @@ static int ltdb_handle_request(struct ldb_module *module,
 
 static int ltdb_init_rootdse(struct ldb_module *module)
 {
-	struct ldb_context *ldb;
-	int ret;
-
-	ldb = ldb_module_get_ctx(module);
-
-	ret = ldb_mod_register_control(module,
-				       LDB_CONTROL_PERMISSIVE_MODIFY_OID);
 	/* ignore errors on this - we expect it for non-sam databases */
+	ldb_mod_register_control(module, LDB_CONTROL_PERMISSIVE_MODIFY_OID);
 
 	/* there can be no module beyond the backend, just return */
 	return LDB_SUCCESS;
