@@ -146,6 +146,19 @@ tdb1_off_t tdb1_find_lock_hash(struct tdb_context *tdb, TDB_DATA key, uint32_t h
 
 static TDB_DATA _tdb1_fetch(struct tdb_context *tdb, TDB_DATA key);
 
+static enum TDB_ERROR tdb_update_hash_cmp(TDB_DATA key, TDB_DATA data, void *private_data)
+{
+	TDB_DATA *dbuf = (TDB_DATA *)private_data;
+
+	if (dbuf->dsize != data.dsize) {
+		return TDB_ERR_EINVAL;
+	}
+	if (memcmp(dbuf->dptr, data.dptr, data.dsize) != 0) {
+		return TDB_ERR_EINVAL;
+	}
+	return TDB_SUCCESS;
+}
+
 /* update an entry in place - this only works if the new data size
    is <= the old data size and the key exists.
    on failure return -1.
@@ -163,18 +176,9 @@ static int tdb1_update_hash(struct tdb_context *tdb, TDB_DATA key, uint32_t hash
 	 * surprisingly common (eg. with a ldb re-index). */
 	if (rec.key_len == key.dsize &&
 	    rec.data_len == dbuf.dsize &&
-	    rec.full_hash == hash) {
-		TDB_DATA data = _tdb1_fetch(tdb, key);
-		if (data.dsize == dbuf.dsize &&
-		    memcmp(data.dptr, dbuf.dptr, data.dsize) == 0) {
-			if (data.dptr) {
-				free(data.dptr);
-			}
+	    rec.full_hash == hash &&
+	    tdb1_parse_record(tdb, key, tdb_update_hash_cmp, &dbuf) == TDB_SUCCESS) {
 			return 0;
-		}
-		if (data.dptr) {
-			free(data.dptr);
-		}
 	}
 
 	/* must be long enough key, data and tailer */
@@ -478,7 +482,6 @@ static int _tdb1_store(struct tdb_context *tdb, TDB_DATA key,
 {
 	struct tdb1_record rec;
 	tdb1_off_t rec_ptr;
-	char *p = NULL;
 	int ret = -1;
 
 	/* check for it existing, on insert. */
@@ -515,20 +518,6 @@ static int _tdb1_store(struct tdb_context *tdb, TDB_DATA key,
 	if (flag != TDB_INSERT)
 		tdb1_delete_hash(tdb, key, hash);
 
-	/* Copy key+value *before* allocating free space in case malloc
-	   fails and we are left with a dead spot in the tdb. */
-
-	if (!(p = (char *)malloc(key.dsize + dbuf.dsize))) {
-		tdb->last_error = tdb_logerr(tdb, TDB_ERR_OOM, TDB_LOG_ERROR,
-					     "tdb1_store: out of memory"
-					     " allocating copy");
-		goto fail;
-	}
-
-	memcpy(p, key.dptr, key.dsize);
-	if (dbuf.dsize)
-		memcpy(p+key.dsize, dbuf.dptr, dbuf.dsize);
-
 	if (tdb->tdb1.max_dead_records != 0) {
 		/*
 		 * Allow for some dead records per hash chain, look if we can
@@ -548,7 +537,10 @@ static int _tdb1_store(struct tdb_context *tdb, TDB_DATA key,
 			if (tdb1_rec_write(tdb, rec_ptr, &rec) == -1
 			    || tdb->tdb1.io->tdb1_write(
 				    tdb, rec_ptr + sizeof(rec),
-				    p, key.dsize + dbuf.dsize) == -1) {
+				    key.dptr, key.dsize) == -1
+			    || tdb->tdb1.io->tdb1_write(
+				    tdb, rec_ptr + sizeof(rec) + key.dsize,
+				    dbuf.dptr, dbuf.dsize) == -1) {
 				goto fail;
 			}
 			goto done;
@@ -591,7 +583,10 @@ static int _tdb1_store(struct tdb_context *tdb, TDB_DATA key,
 
 	/* write out and point the top of the hash chain at it */
 	if (tdb1_rec_write(tdb, rec_ptr, &rec) == -1
-	    || tdb->tdb1.io->tdb1_write(tdb, rec_ptr+sizeof(rec), p, key.dsize+dbuf.dsize)==-1
+	    || tdb->tdb1.io->tdb1_write(tdb, rec_ptr + sizeof(rec),
+					key.dptr, key.dsize) == -1
+	    || tdb->tdb1.io->tdb1_write(tdb, rec_ptr + sizeof(rec) + key.dsize,
+					dbuf.dptr, dbuf.dsize) == -1
 	    || tdb1_ofs_write(tdb, TDB1_HASH_TOP(hash), &rec_ptr) == -1) {
 		/* Need to tdb1_unallocate() here */
 		goto fail;
@@ -603,8 +598,6 @@ static int _tdb1_store(struct tdb_context *tdb, TDB_DATA key,
 	if (ret == 0) {
 		tdb1_increment_seqnum(tdb);
 	}
-
-	SAFE_FREE(p);
 	return ret;
 }
 
