@@ -1,5 +1,6 @@
-#include <ccan/tdb2/tdb2.h>
-#include <ccan/tap/tap.h>
+#include "config.h"
+#include "tdb2.h"
+#include "tap-interface.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
