@@ -1,22 +1,11 @@
-#include <ccan/tdb2/private.h>
+#include "private.h"
 #include <ccan/failtest/failtest_override.h>
 #include "tdb2-source.h"
-#include <ccan/tap/tap.h>
+#include "tap-interface.h"
 #include <ccan/failtest/failtest.h>
 #include "logging.h"
 #include "failtest_helper.h"
 
-static bool failtest_suppress = false;
-
-/* Don't need to test everything here, just want expand testing. */
-static enum failtest_result
-suppress_failure(struct failtest_call *history, unsigned num)
-{
-	if (failtest_suppress)
-		return FAIL_DONT_FAIL;
-	return block_repeat_failures(history, num);
-}
-
 int main(int argc, char *argv[])
 {
 	unsigned int i;
@@ -32,7 +21,7 @@ int main(int argc, char *argv[])
 	struct tdb_data data = tdb_mkdata("data", 4);
 
 	failtest_init(argc, argv);
-	failtest_hook = suppress_failure;
+	failtest_hook = block_repeat_failures;
 	failtest_exit_check = exit_check_log;
 
 	failtest_suppress = true;
