@@ -1,7 +1,7 @@
 /* We should be able to tdb_check a O_RDONLY tdb, and we were previously allowed
  * to tdb_check() inside a transaction (though that's paranoia!). */
 #include "tdb2-source.h"
-#include <ccan/tap/tap.h>
+#include "tap-interface.h"
 #include <stdlib.h>
 #include <err.h>
 #include "logging.h"
@@ -22,10 +22,8 @@ int main(int argc, char *argv[])
 		       O_CREAT|O_TRUNC|O_RDWR, 0600, &hsize);
 
 	ok1(tdb);
-	key.dsize = strlen("hi");
-	key.dptr = (void *)"hi";
-	data.dsize = strlen("world");
-	data.dptr = (void *)"world";
+	key = tdb_mkdata("hi", strlen("hi"));
+	data = tdb_mkdata("world", strlen("world"));
 
 	ok1(tdb_store(tdb, key, data, TDB_INSERT) == TDB_SUCCESS);
 	ok1(tdb_check(tdb, NULL, NULL) == TDB_SUCCESS);
