@@ -707,16 +707,20 @@ static void py_ldb_debug(void *context, enum ldb_debug_level level, const char *
 	PyObject_CallFunction(fn, discard_const_p(char, "(i,O)"), level, PyString_FromFormatV(fmt, ap));
 }
 
-static PyObject *py_ldb_set_debug(PyLdbObject *self, PyObject *args)
+static PyObject *py_ldb_set_debug(PyObject *self, PyObject *args)
 {
 	PyObject *cb;
+	struct ldb_context *ldb_ctx;
 
 	if (!PyArg_ParseTuple(args, "O", &cb))
 		return NULL;
 
 	Py_INCREF(cb);
 	/* FIXME: Where do we DECREF cb ? */
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_set_debug(self->ldb_ctx, py_ldb_debug, cb), pyldb_Ldb_AsLdbContext(self));
+	ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError,
+		ldb_set_debug(ldb_ctx, py_ldb_debug, cb),
+		ldb_ctx);
 
 	Py_RETURN_NONE;
 }
@@ -745,31 +749,46 @@ static PyObject *py_ldb_set_modules_dir(PyTypeObject *self, PyObject *args)
 
 static PyObject *py_ldb_transaction_start(PyLdbObject *self)
 {
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_transaction_start(pyldb_Ldb_AsLdbContext(self)), pyldb_Ldb_AsLdbContext(self));
+	struct ldb_context *ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	int ldb_err;
+	ldb_err = ldb_transaction_start(ldb_ctx);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_err, ldb_ctx);
 	Py_RETURN_NONE;
 }
 
 static PyObject *py_ldb_transaction_commit(PyLdbObject *self)
 {
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_transaction_commit(pyldb_Ldb_AsLdbContext(self)), pyldb_Ldb_AsLdbContext(self));
+	struct ldb_context *ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	int ldb_err;
+	ldb_err = ldb_transaction_commit(ldb_ctx);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_err, ldb_ctx);
 	Py_RETURN_NONE;
 }
 
 static PyObject *py_ldb_transaction_prepare_commit(PyLdbObject *self)
 {
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_transaction_prepare_commit(pyldb_Ldb_AsLdbContext(self)), pyldb_Ldb_AsLdbContext(self));
+	struct ldb_context *ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	int ldb_err;
+	ldb_err = ldb_transaction_prepare_commit(ldb_ctx);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_err, ldb_ctx);
 	Py_RETURN_NONE;
 }
 
 static PyObject *py_ldb_transaction_cancel(PyLdbObject *self)
 {
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_transaction_cancel(pyldb_Ldb_AsLdbContext(self)), pyldb_Ldb_AsLdbContext(self));
+	struct ldb_context *ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	int ldb_err;
+	ldb_err = ldb_transaction_cancel(ldb_ctx);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_err, ldb_ctx);
 	Py_RETURN_NONE;
 }
 
 static PyObject *py_ldb_setup_wellknown_attributes(PyLdbObject *self)
 {
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_setup_wellknown_attributes(pyldb_Ldb_AsLdbContext(self)), pyldb_Ldb_AsLdbContext(self));
+	struct ldb_context *ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	int ldb_err;
+	ldb_err = ldb_setup_wellknown_attributes(ldb_ctx);
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ldb_err, ldb_ctx);
 	Py_RETURN_NONE;
 }
 
@@ -905,6 +924,7 @@ static PyObject *py_ldb_connect(PyLdbObject *self, PyObject *args, PyObject *kwa
 	int ret;
 	const char **options;
 	const char * const kwnames[] = { "url", "flags", "options", NULL };
+	struct ldb_context *ldb_ctx;
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zIO",
 					 discard_const_p(char *, kwnames),
@@ -919,10 +939,11 @@ static PyObject *py_ldb_connect(PyLdbObject *self, PyObject *args, PyObject *kwa
 			return NULL;
 	}
 
-	ret = ldb_connect(pyldb_Ldb_AsLdbContext(self), url, flags, options);
+	ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	ret = ldb_connect(ldb_ctx, url, flags, options);
 	talloc_free(options);
 
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, pyldb_Ldb_AsLdbContext(self));
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
 
 	Py_RETURN_NONE;
 }
@@ -970,7 +991,7 @@ static PyObject *py_ldb_modify(PyLdbObject *self, PyObject *args, PyObject *kwar
 	if (validate) {
 		ret = ldb_msg_sanity_check(ldb_ctx, msg);
 		if (ret != LDB_SUCCESS) {
-			PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+			PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
 			talloc_free(mem_ctx);
 			return NULL;
 		}
@@ -990,7 +1011,8 @@ static PyObject *py_ldb_modify(PyLdbObject *self, PyObject *args, PyObject *kwar
 	ret = ldb_transaction_start(ldb_ctx);
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
+		return NULL;
 	}
 
 	ret = ldb_request(ldb_ctx, req);
@@ -1051,7 +1073,7 @@ static struct ldb_message *PyDict_AsMessage(TALLOC_CTX *mem_ctx,
 
 	while (PyDict_Next(py_obj, &dict_pos, &key, &value)) {
 		char *key_str = PyString_AsString(key);
-		if (strcmp(key_str, "dn") != 0) {
+		if (ldb_attr_cmp(key_str, "dn") != 0) {
 			msg_el = PyObject_AsMessageElement(msg->elements, value,
 							   mod_flags, key_str);
 			if (msg_el == NULL) {
@@ -1117,7 +1139,7 @@ static PyObject *py_ldb_add(PyLdbObject *self, PyObject *args, PyObject *kwargs)
 
 	ret = ldb_msg_sanity_check(ldb_ctx, msg);
 	if (ret != LDB_SUCCESS) {
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
 		talloc_free(mem_ctx);
 		return NULL;
 	}
@@ -1136,13 +1158,14 @@ static PyObject *py_ldb_add(PyLdbObject *self, PyObject *args, PyObject *kwargs)
 	ret = ldb_transaction_start(ldb_ctx);
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
+		return NULL;
 	}
 
 	ret = ldb_request(ldb_ctx, req);
 	if (ret == LDB_SUCCESS) {
 		ret = ldb_wait(req->handle, LDB_WAIT_ALL);
-	} 
+	}
 
 	if (ret == LDB_SUCCESS) {
 		ret = ldb_transaction_commit(ldb_ctx);
@@ -1207,7 +1230,8 @@ static PyObject *py_ldb_delete(PyLdbObject *self, PyObject *args, PyObject *kwar
 	ret = ldb_transaction_start(ldb_ctx);
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
+		return NULL;
 	}
 
 	ret = ldb_request(ldb_ctx, req);
@@ -1286,7 +1310,8 @@ static PyObject *py_ldb_rename(PyLdbObject *self, PyObject *args, PyObject *kwar
 	ret = ldb_transaction_start(ldb_ctx);
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
+		return NULL;
 	}
 
 	ret = ldb_request(ldb_ctx, req);
@@ -1322,12 +1347,15 @@ static PyObject *py_ldb_schema_attribute_add(PyLdbObject *self, PyObject *args)
 	char *attribute, *syntax;
 	unsigned int flags;
 	int ret;
+	struct ldb_context *ldb_ctx;
+
 	if (!PyArg_ParseTuple(args, "sIs", &attribute, &flags, &syntax))
 		return NULL;
 
-	ret = ldb_schema_attribute_add(pyldb_Ldb_AsLdbContext(self), attribute, flags, syntax);
+	ldb_ctx = pyldb_Ldb_AsLdbContext(self);
+	ret = ldb_schema_attribute_add(ldb_ctx, attribute, flags, syntax);
 
-	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, pyldb_Ldb_AsLdbContext(self));
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
 
 	Py_RETURN_NONE;
 }
@@ -1565,7 +1593,7 @@ static PyObject *py_ldb_search(PyLdbObject *self, PyObject *args, PyObject *kwar
 
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
 		return NULL;
 	}
 
@@ -1579,7 +1607,7 @@ static PyObject *py_ldb_search(PyLdbObject *self, PyObject *args, PyObject *kwar
 
 	if (ret != LDB_SUCCESS) {
 		talloc_free(mem_ctx);
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb_ctx);
+		PyErr_SetLdbError(PyExc_LdbError, ret, ldb_ctx);
 		return NULL;
 	}
 
@@ -1649,10 +1677,8 @@ static PyObject *py_ldb_sequence_number(PyLdbObject *self, PyObject *args)
 
 	ret = ldb_sequence_number(ldb, type, &value);
 
-	if (ret != LDB_SUCCESS) {
-		PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb);
-		return NULL;
-	}
+	PyErr_LDB_ERROR_IS_ERR_RAISE(PyExc_LdbError, ret, ldb);
+
 	return PyLong_FromLongLong(value);
 }
 static PyMethodDef py_ldb_methods[] = {
@@ -2001,7 +2027,7 @@ static PyObject *py_ldb_module_search(PyLdbModuleObject *self, PyObject *args, P
 
 	talloc_free(req);
 
-	return py_ret;	
+	return py_ret;
 }
 
 
@@ -2117,6 +2143,7 @@ static PyTypeObject PyLdbModule = {
 	.tp_basicsize = sizeof(PyLdbModuleObject),
 	.tp_dealloc = (destructor)py_ldb_module_dealloc,
 	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_doc = "LDB module (extension)",
 };
 
 
@@ -2436,6 +2463,7 @@ static PyTypeObject PyLdbMessageElement = {
 	.tp_as_sequence = &py_ldb_msg_element_seq,
 	.tp_new = py_ldb_msg_element_new,
 	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_doc = "An element of a Message",
 };
 
 
@@ -2514,7 +2542,7 @@ static PyObject *py_ldb_msg_getitem_helper(PyLdbMessageObject *self, PyObject *p
 		return NULL;
 	}
 	name = PyString_AsString(py_name);
-	if (!strcmp(name, "dn"))
+	if (!ldb_attr_cmp(name, "dn"))
 		return pyldb_Dn_FromDn(msg->dn);
 	el = ldb_msg_find_element(msg, name);
 	if (el == NULL) {
@@ -2666,12 +2694,17 @@ static int py_ldb_msg_setitem(PyLdbMessageObject *self, PyObject *name, PyObject
 		/* delitem */
 		ldb_msg_remove_attr(self->msg, attr_name);
 	} else {
+		int ret;
 		struct ldb_message_element *el = PyObject_AsMessageElement(self->msg,
 									   value, 0, attr_name);
 		if (el == NULL)
 			return -1;
 		ldb_msg_remove_attr(pyldb_Message_AsMessage(self), attr_name);
-		ldb_msg_add(pyldb_Message_AsMessage(self), el, el->flags);
+		ret = ldb_msg_add(pyldb_Message_AsMessage(self), el, el->flags);
+		if (ret != LDB_SUCCESS) {
+			PyErr_SetLdbError(PyExc_LdbError, ret, NULL);
+			return -1;
+		}
 	}
 	return 0;
 }
@@ -2836,6 +2869,7 @@ static PyTypeObject PyLdbMessage = {
 	.tp_flags = Py_TPFLAGS_DEFAULT,
 	.tp_iter = (getiterfunc)py_ldb_msg_iter,
 	.tp_compare = (cmpfunc)py_ldb_msg_compare,
+	.tp_doc = "A LDB Message",
 };
 
 static PyObject *PyLdbTree_FromTree(struct ldb_parse_tree *tree)
@@ -2864,6 +2898,7 @@ static PyTypeObject PyLdbTree = {
 	.tp_basicsize = sizeof(PyLdbTreeObject),
 	.tp_dealloc = (destructor)py_ldb_tree_dealloc,
 	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_doc = "A search tree",
 };
 
 /* Ldb_module */
@@ -3247,24 +3282,25 @@ static PyObject *py_binary_decode(PyObject *self, PyObject *args)
 
 static PyMethodDef py_ldb_global_methods[] = {
 	{ "register_module", py_register_module, METH_VARARGS, 
-		"S.register_module(module) -> None\n"
+		"S.register_module(module) -> None\n\n"
 		"Register a LDB module."},
 	{ "timestring", py_timestring, METH_VARARGS, 
-		"S.timestring(int) -> string\n"
+		"S.timestring(int) -> string\n\n"
 		"Generate a LDAP time string from a UNIX timestamp" },
 	{ "string_to_time", py_string_to_time, METH_VARARGS,
-		"S.string_to_time(string) -> int\n"
+		"S.string_to_time(string) -> int\n\n"
 		"Parse a LDAP time string into a UNIX timestamp." },
 	{ "valid_attr_name", py_valid_attr_name, METH_VARARGS,
-		"S.valid_attr_name(name) -> bool\n"
+		"S.valid_attr_name(name) -> bool\n\nn"
 		"Check whether the supplied name is a valid attribute name." },
 	{ "open", (PyCFunction)py_ldb_new, METH_VARARGS|METH_KEYWORDS,
-		NULL },
+		"S.open() -> Ldb\n\n"
+		"Open a new LDB context." },
 	{ "binary_encode", py_binary_encode, METH_VARARGS,
-		"S.binary_encode(string) -> string\n"
+		"S.binary_encode(string) -> string\n\n"
 		"Perform a RFC2254 binary encoding on a string" },
 	{ "binary_decode", py_binary_decode, METH_VARARGS,
-		"S.binary_decode(string) -> string\n"
+		"S.binary_decode(string) -> string\n\n"
 		"Perform a RFC2254 binary decode on a string" },
 	{ NULL }
 };
